import { IQueryHandler, QueryHandler } from '@nestjs/cqrs';
import { GetConfigurationQuery } from './get-configuration.query';
import { Inject } from '@nestjs/common';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';
import { ConfigurationRepositoryPort } from '../../ports/configuration.repository.port';
import { ConfigurationDomain } from '@modules/configuration/core/domain/configuration.types';

@QueryHandler(GetConfigurationQuery)
export class GetConfigurationQueryHandler implements IQueryHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: ConfigurationRepositoryPort,
  ) {}
  async execute(query: GetConfigurationQuery): Promise<ConfigurationEntity> {
    return await this.configurationRepository.get({
      domain: query.domain as ConfigurationDomain,
      key: query.key,
    });
  }
}
