import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { Inject, Injectable, OnApplicationBootstrap } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { ConfigurationRepositoryPort } from '../ports/configuration.repository.port';
import {
  CarpoolConfig,
  ConfigurationDomain,
  PaginationConfig,
} from '../../domain/configuration.types';
import { NotFoundException } from '@mobicoop/ddd-library';

@Injectable()
export class PopulateService implements OnApplicationBootstrap {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: ConfigurationRepositoryPort,
    private readonly configService: ConfigService,
  ) {}

  onApplicationBootstrap() {
    this._populate();
  }

  private _populate = async (): Promise<void> => {
    const carpoolConfig: CarpoolConfig = this.configService.get<CarpoolConfig>(
      'carpool',
    ) as CarpoolConfig;
    const paginationConfig: PaginationConfig =
      this.configService.get<PaginationConfig>(
        'pagination',
      ) as PaginationConfig;
    await Promise.all([
      this._populateConfig(ConfigurationDomain.CARPOOL, carpoolConfig),
      this._populateConfig(ConfigurationDomain.PAGINATION, paginationConfig),
    ]);
  };

  private _populateConfig = async <T>(
    domain: ConfigurationDomain,
    config: T,
  ): Promise<void> => {
    let key: keyof typeof config;
    for (key in config) {
      try {
        await this.configurationRepository.get({
          domain,
          key,
        });
      } catch (error: any) {
        if (error instanceof NotFoundException) {
          this.configurationRepository.set(
            {
              domain,
              key,
            },
            `${config[key]}`,
          );
        }
      }
    }
  };
}
