import { ConfigurationMapper } from '@modules/configuration/configuration.mapper';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';
import { ConfigurationDomain } from '@modules/configuration/core/domain/configuration.types';
import {
  ConfigurationReadModel,
  ConfigurationWriteModel,
} from '@modules/configuration/infrastructure/configuration.repository';
import { ConfigurationResponseDto } from '@modules/configuration/interface/dtos/configuration.response.dto';
import { Test } from '@nestjs/testing';

const now = new Date('2023-06-21 06:00:00');
const configurationEntity: ConfigurationEntity = new ConfigurationEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df44',
  props: {
    identifier: {
      domain: ConfigurationDomain.CARPOOL,
      key: 'seatsProposed',
    },
    value: '3',
  },
  createdAt: now,
  updatedAt: now,
});
const configurationReadModel: ConfigurationReadModel = {
  key: 'AD:seatsProposed',
  value: '4',
};

describe('Configuration Mapper', () => {
  let configurationMapper: ConfigurationMapper;

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      providers: [ConfigurationMapper],
    }).compile();
    configurationMapper = module.get<ConfigurationMapper>(ConfigurationMapper);
  });

  it('should be defined', () => {
    expect(configurationMapper).toBeDefined();
  });

  it('should map domain entity to persistence data', async () => {
    const mapped: ConfigurationWriteModel =
      configurationMapper.toPersistence(configurationEntity);
    expect(mapped.value).toBe('3');
  });

  it('should map persisted data to domain entity', async () => {
    const mapped: ConfigurationEntity = configurationMapper.toDomain(
      configurationReadModel,
    );
    expect(mapped.getProps().value).toBe('4');
  });

  it('should map domain entity to response', async () => {
    const mapped: ConfigurationResponseDto =
      configurationMapper.toResponse(configurationEntity);
    expect(mapped.id).toBe('c160cf8c-f057-4962-841f-3ad68346df44');
  });
});
