import { Test, TestingModule } from '@nestjs/testing';
import { SetConfigurationRequestDto } from '@modules/configuration/interface/grpc-controllers/dtos/set-configuration.request.dto';
import { ConfigurationDomain } from '@modules/configuration/core/domain/configuration.types';
import { SetConfigurationService } from '@modules/configuration/core/application/commands/set-configuration/set-configuration.service';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { SetConfigurationCommand } from '@modules/configuration/core/application/commands/set-configuration/set-configuration.command';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';

const setConfigurationRequest: SetConfigurationRequestDto = {
  domain: ConfigurationDomain.CARPOOL,
  key: 'seatsProposed',
  value: '3',
};

const mockConfigurationRepository = {
  set: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('Set Configuration Service', () => {
  let setConfigurationService: SetConfigurationService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        SetConfigurationService,
      ],
    }).compile();

    setConfigurationService = module.get<SetConfigurationService>(
      SetConfigurationService,
    );
  });

  it('should be defined', () => {
    expect(setConfigurationService).toBeDefined();
  });

  describe('execution', () => {
    const setConfigurationCommand = new SetConfigurationCommand(
      setConfigurationRequest,
    );
    it('should set an existing configuration item', async () => {
      jest.spyOn(mockConfigurationRepository, 'set');
      ConfigurationEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await setConfigurationService.execute(setConfigurationCommand);
      expect(mockConfigurationRepository.set).toHaveBeenCalledTimes(1);
    });
    it('should throw an error if something bad happens on configuration item update', async () => {
      ConfigurationEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        setConfigurationService.execute(setConfigurationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
